@@ -1,6 +1,6 @@
 # Copyright (c) 2010 Infrae. All rights reserved.
 # See also LICENSE.txt
-# $Id: zope2.py 45562 2010-09-28 17:01:44Z sylvain $
+# $Id$
 
 import os
 
@@ -8,6 +8,7 @@ from AccessControl.SecurityManagement import newSecurityManager
 from AccessControl.SecurityManagement import noSecurityManager
 from App.class_init import InitializeClass
 from OFS.Application import get_folder_permissions, get_products
+from OFS.metaconfigure import get_packages_to_initialize
 from OFS.Application import install_product, install_package, AppInitializer
 from OFS.Folder import Folder
 from Testing import ZopeTestCase
@@ -126,12 +127,10 @@ class TestAppInitializer(AppInitializer):
         """Zope 2 install a given package.
         """
         if name not in self.installed_packages:
-            for module, init_func in getattr(
-                Products, '_packages_to_initialize', []):
+            for module, init_func in get_packages_to_initialize():
                 if module.__name__ == name:
                     install_package(app, module, init_func, raise_exc=1)
                     self.installed_packages.append(name)
-                    Products._packages_to_initialize.remove((module, init_func))
                     break
 
 
@@ -210,15 +209,24 @@ class Zope2Layer(ZCMLLayer):
         transaction.savepoint()
 
         # Initialize the "application"
-        TestAppInitializer(
-            app, self.products, self.packages, self.users).initialize()
-        self._install_application(makerequest(
-                app, environ={'SERVER_NAME': 'localhost'}))
-
-        # Close
-        transaction.commit()
-        connection.close()
-        noSecurityManager()
+        try:
+            TestAppInitializer(
+                app, self.products, self.packages, self.users).initialize()
+            self._install_application(makerequest(
+                    app, environ={'SERVER_NAME': 'localhost'}))
+        except Exception as error:
+            # There was an error during the application 'setUp'. Abort
+            # the transaction and continue, otherwise test in other
+            # layers might fail because of this failure.
+            transaction.abort()
+            raise error
+        else:
+            # Close
+            transaction.commit()
+        finally:
+            # In any case, close the connection and continue
+            connection.close()
+            noSecurityManager()
 
     def _reload_zope(self, db):
         """Need to re-mount/re-setup SESSIONs if we just reload a Zope
